/*
 * MyCanvasBOM.java
 *
 * Created on 15 de abril de 2005, 12:58
 */

package learningbom;

import BOM.paint.PaintText;
import BOM.BackwardsOracleMatching;
import oracle.FactorOracle;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.util.ArrayList;
import javax.swing.JPanel;


/**
 *
 * @author Alejandro Touza Paredes
 */
public class MyCanvasBOM extends MyCanvas{
    
    private final static String COMMON_USED_SIGNS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.,;:<>?![]+-*/\\ \"";
    /** BOM que se mostrar en el Canvas. */
    private BackwardsOracleMatching bom;
    private FactorOracle fo;
    /** Parmetros con los que se mostrar BOM. */
    private PaintText pt;
    /** Indica si el botn izquierdo se ha pulsado dentro del Canvas. */
    private boolean leftButtonClicked; 
    /** Indica si se ha movido el dibujo del canvas antes de repintar. De este modo 
     *  pintamos la sombra del rectngulo y el fondo solo cuando es necesario. */
    private boolean moving; 
    /** Indica el ltimo punto donde se ha producido un evento de ratn con el
     *  botn izquierdo pulsado. */
    private Point pointLeftClick; 
    /**  x = Paso actual bomList(currentStep.x). <br>
     *   y = Subpaso dentro de CurrentStep. 
     *       0) Pintado rectngulo que indica la posicin.
     *       1-bomList(x).y) Pintado de bsqueda de caracteres. 
     *       bomList(x).y+1) Pintado de ractngulo de resolucin (pattern encontrado, 
     *       pattern no encontrado) y (opcional) carcter no encontrado.*/
    private Point currentStep;
    private String pattern;
    private ArrayList bomList;
    private Dimension backgroundRectangleSize;
    private boolean fitToScreen;
   
    //CONSTRUCTORAS
    public MyCanvasBOM(JPanel jpanel){
        
        super(jpanel);
        
        resetParameters();
        fitToScreen = false;
        pt = new PaintText();
        setBackground(pt.getBackgroundColor());
        initGraphics2D();
    }
    ///////////////////////////////////////////////////////////////////////////////////
    // EVENTOS DE RATN
    /** Registra si se ha producido un click del botn izquierdo y si es as 
     *  recoge el punto donde se ha producido.
     *  @param evt Evento de ratn. */
    protected void mousePressedEvent(java.awt.event.MouseEvent evt){
                
        if(start && evt.getButton()==evt.BUTTON1) //Botn Izqdo.
        {
            leftButtonClicked = true;
            pointLeftClick = evt.getPoint();
        }
    }
    protected void mouseReleasedEvent(java.awt.event.MouseEvent evt){
                
        if(evt.getButton()==evt.BUTTON1)
        {    
            leftButtonClicked = false;
        }
    }
    /** Mueve el Factor Oracle en las direcciones marcadas por el movimiento del
     *  ratn con el botn izquierdo pulsado.
     *  @param evt Evento de ratn. */
     protected void mouseDraggedEvent(java.awt.event.MouseEvent evt){
                            
        if(leftButtonClicked)
        {
            this.moveFromTextStartPoint(evt.getPoint().x - pointLeftClick.x, evt.getPoint().y - pointLeftClick.y);
            pointLeftClick = evt.getPoint();
            //pt.setTextStartPoint(p);
            repaint();
        }
    }
    ////////////////////////////////////////////////////////////////////////////////////
    // EVENTOS DE COMPONENTES
    /** Se encarga de hacer un resize del canvas y de adaptar el autmata a el nuevo
     *  tamao de la ventana */
     protected void componentResizedEvent(java.awt.event.ComponentEvent evt){
        
        super.componentResizedEvent(evt);
        initGraphics2D();
        if(fitToScreen) fitToScreen();
        moving = true;
        repaint();   
    }
     
    //////////////
    public boolean getFitToScreen(){
        
        return fitToScreen;
    }
    public void setFitToScreen(boolean b){
        
        fitToScreen = b;
    }
     
    ////////////////////////////////////////////////////////////////////////////////////
    //FUNCIONES DE INICIALIZADO / PINTADO DEL GRAPH ORACLE
    /** Crea una instancia de Factor Oracle con pattern. Pinta los nodos vacos 
     *  del autmata si asi se le indica.
     *  @param pattern String con el que crea una instancia de Factor Oracle.
     *  @param b Booleano que indica si debemos pintar el autmata tras inicializarlo.  */
    public void start(String text, String pattern){
        
        resetParameters();
        
        this.start = true;
        this.pattern = pattern;
                
        bom = new BackwardsOracleMatching(pattern, text);
        // Pasamos el ArrayList donde obtendremos el resultado de bom.doBOM()
        bomList = new ArrayList();
        bom.doBOM(bomList);
        
        // Nos guardamos un GraphOracle para uso propio
        fo = new FactorOracle(new StringBuffer(pattern).reverse().toString());
        fo.Oracle();
        
        //Inicializamos la posicin de nuestro tablero BOM
        pt.setTextStartPoint(getInitialPoint());
        pt.setTextGap(2, 6);
        setBackgroundRectangleSize(); 
        pt.calculateTextPaintingDimension(g2d, COMMON_USED_SIGNS);
    }
    public void reset(){
        
        if(start)
        {
            resetParameters();
        
            //Inicializamos la posicin de nuestro tablero BOM
            pt.setTextStartPoint(getInitialPoint());
            pt.setTextGap(2, 6);
            //setBackgroundRectangleSize();
        }
    }
    /** Retrocede un subpaso el algoritmo que se pintar en MyCanvasBOM.
     *  @return <b>true</b> si an puede retroceder ms pasos. */
    public boolean stepBack(){
        
        if (currentStep.y>0) currentStep.y--; 
        else if(currentStep.y == 0 || (currentStep.y == -1 && currentStep.x == bomList.size()))
        {
            if(currentStep.x==0)
            {
               currentStep.y = -1;
               return false;
            }
            else
            {
               currentStep.x--;
               Point p = (Point)(bomList.get(currentStep.x));
               currentStep.y = p.y + 1;
               
               //Comprobamos si el primer carcter de la secuencia a explorar en el
               //currentStep cae dentro de nuestra ventana
               int aux = (pt.getTextStartPoint().x + (p.x)*pt.getCellSize().width);
               if( aux < 0)
               {
                   aux = -aux + getSize().width/4;
                   //comprobamos que no vayamos ms all del principio
                   if(pt.getTextStartPoint().x + aux > getInitialPoint().x)
                   {
                       moveTextToPoint(getInitialPoint().x, pt.getTextStartPoint().y);
                       
                   }
                   else moveFromTextStartPoint(aux,0);
               }
               
            }
        }
        return true;
    }
    /** Avanza un subpaso el algoritmo que se pintar en MyCanvasBOM.
     *  @return <b>true</b> si an quedan ms pasos a realizar. */
    public boolean step(){
        
        //Recogemos el punto actual
        Point p = (Point)(bomList.get(currentStep.x));
        
        if(p.y >= currentStep.y) currentStep.y++;
        else if (++currentStep.x == bomList.size())
        {  
            currentStep.y = -1;
            return false;
        }
        else
        { 
            currentStep.y = 0; //ya hemos aumentado currentStep.x en la comparacin anterior
            
            //Aqu podramos desplazar el canvas para que se pueda ir siguiendo el algoritmo
            //sin necesidad de andar con las flechas.
            p = (Point)(bomList.get(currentStep.x));
            int aux = (pt.getTextStartPoint().x + (p.x + pattern.length())*pt.getCellSize().width);
            aux -= getSize().width;
            if( aux > 0)
            {
                moveFromTextStartPoint(-aux + pattern.length()*pt.getCellSize().width
                                       - (getSize().width*3/4),0);
            }
        }
        return true;
    }
    /** Modifica currentStep para que se pinte todo MyCanvasBOM.*/   
    public void complete(){
        
        currentStep.x = bomList.size();
        currentStep.y = -1;
    }
    /** Aumenta el tamao de la fuente usada por MyCanvasBOM.
     *  @return El tamao de la fuente tras aplicar la funcin. */
    public int makeItBigger(){
        
        Font f =  pt.getTextFont();
        int oldFontSize = f.getSize();
        f = f.deriveFont((float)(oldFontSize+1));
        double fontWidthInPixels = (f.getSize()*pt.getTextPaintingDimension().width)/oldFontSize;
     
        pt.setTextFont(f);
        g2d.setFont(f); //Ponemos esto para poder calcular el ascent de la fuente
        pt.setTextPaintingDimension((int)fontWidthInPixels, 
                                    g2d.getFontMetrics().getAscent() -  
                                    g2d.getFontMetrics().getDescent());
        setBackgroundRectangleSize();
        moving = true;
        return f.getSize();
    }
    /** Disminuye el tamao de la fuente usada por MyCanvasBOM hasta un mnimo de 1.
     *  @return El tamao de la fuente tras aplicar la funcin. */ 
    public int makeItSmaller(){
        
        Font f =  pt.getTextFont();
        
        if(f.getSize()==1) return -1;
        
        int oldFontSize = f.getSize();
        f = f.deriveFont((float)(oldFontSize-1));
        double fontWidthInPixels = (f.getSize()*pt.getTextPaintingDimension().width)/oldFontSize;
     
        pt.setTextFont(f);
        g2d.setFont(f); //Ponemos esto para poder calcular el ascent de la fuente
        pt.setTextPaintingDimension((int)fontWidthInPixels, 
                                    g2d.getFontMetrics().getAscent() -  
                                    g2d.getFontMetrics().getDescent());
        setBackgroundRectangleSize();
        moving = true;
        return f.getSize();
    }
    /** Asigna el tamao dado a la fuente usada por MyCanvasBOM.
     *  @param size Tamao que se asignar a la fuente. */
    public void resizeBOM(int size){
        
        Font f =  pt.getTextFont();
        int oldFontSize = f.getSize();
        f = f.deriveFont((float)size);
        double fontWidthInPixels = (f.getSize()*pt.getTextPaintingDimension().width)/oldFontSize;
     
        pt.setTextFont(f);
        g2d.setFont(f); //Ponemos esto para poder calcular el ascent de la fuente
        pt.setTextPaintingDimension((int)fontWidthInPixels, 
                                    g2d.getFontMetrics().getAscent() -  
                                    g2d.getFontMetrics().getDescent());
        setBackgroundRectangleSize();
        moving = true;
   }
    /** Escala MyCanvasBOM para un visualizado completo en pantalla. Si el texto resultante
     *  de dicho escalado resultase <15 se utilizara 15 como tamao de fuente.
     *  @return El tamao de la fuente resultante del escalado. */
    public int fitToScreen(){
     /********* PRUEBAS, QUITAMOS EL CENTRAR EN EL FIT TO SCREEN PA QUE NO LO HAGA EN EL RESIZE ***/   
     //pt.setTextStartPoint(getInitialPoint());
     /***********************************************************/
        //Calculamos el tamao en puntos de nuestra fuente para que quepa de la mejor forma en 
        //nuestro canvas (tenemos en cuenta la relacion puntos/pxels de fontRatio)
        // fontWidthInPixels: anchura mxima en pxels de cada carcter para que quepa todo el texto
        // pt.getFontRatio(): Equivalencia puntos/pixels
        // 15 -> tamao mnimo de fuente que consideramos a mostrar
        // 50 -> extra para los bordes del rectngulo
        double fontWidthInPixels = 
                Math.max(((getSize().width - 50)/bom.getText().length()) - pt.getTextGap().width, 15);
        int fontSize = (int)(fontWidthInPixels * pt.getFontRatio());
        
        Font f = pt.getTextFont().deriveFont((float)fontSize);
       
        pt.setTextFont(f);
        g2d.setFont(f); //Ponemos la fuente para poder calcular su altura 
        pt.setTextPaintingDimension((int)fontWidthInPixels, g2d.getFontMetrics().getAscent() -  
                                                            g2d.getFontMetrics().getDescent());
        setBackgroundRectangleSize();
        moving = true;
        return fontSize;
    }
    /** Calcula el tamao que ha de tener el rectngulo de fondo para que quepan todo el texto + 
     *  los mrgenes laterales, superiores e inferiores. */
    private void setBackgroundRectangleSize(){
        
        // 30, 30 es el Gap existente
        int width  = bom.getText().length()*(pt.getTextPaintingDimension().width + pt.getTextGap().width) + 30;
        int height = (bomList.size()+1)*(pt.getTextPaintingDimension().height + pt.getTextGap().height) + 30;
        backgroundRectangleSize = new Dimension(width, height);
    }
    /** Mueve MyCanvasOracle al punto inicial por defecto. */
    public void moveToDefaultStartPoint(){
        
        pt.setTextStartPoint(getInitialPoint());
        moving = true;
    }
    /** Desplaza x e y MyCanvasOracle desde el punto inicial por defecto. 
     *  @param x Desplazamiento en el eje x.
     *  @param y Desplazamiento en el eje y. */
    public void moveFromTextStartPoint(int x, int y){
        
        pt.setTextStartPoint(new Point(pt.getTextStartPoint().x + x, 
                                       pt.getTextStartPoint().y + y));
        moving = true;
    }
    /** Desplaza x e y MyCanvasOracle desde el punto inicial por defecto. 
     *  @param x Desplazamiento en el eje x.
     *  @param y Desplazamiento en el eje y. */
    public void moveTextToPoint(int x, int y){
        
        pt.setTextStartPoint(new Point(x,y)); 
        moving = true;
    }
    /** Retorna el String que se est explorando en el momento. Viene dado por currentStep. 
      * El String es enviado en el sentido de lectura de derecha a izquierda, ya que es esto lo
      * que busca BOM. */
    public String getCurrentString(){
        
        if(currentStep.y != -1)
        {
            int endCol = ((Point)(bomList.get(currentStep.x))).x + pattern.length();
            return (new StringBuffer(
                        bom.getText().substring(endCol - currentStep.y, endCol)
                        )).reverse().toString();
        }
        return "";
    }
    public String getText(){
        
        return bom.getText();
    }
    /** Pinta MyCanvasBOM teniendo en cuenta currentStep esto es, el paso y subpaso actual. */
    public void paint(Graphics g){
    
        if(!start)return;
        /* Ahora mismo no hacemos uso del Graphics pasado, si alguien quiere que su Graphics
         * sea usado habr de hacer uso de l */
        cleanBackground(pt.getBackgroundColor());
        g2d.setStroke(new BasicStroke(1.0f));
        // 1 PINTAMOS LA TABLA QUE RODEA AL TEXTO
        pt.drawTable(g2d, 0, 0, 1, bom.getText().length());
        // 2 PINTAMOS EL TEXTO COMO INACTIVO
        pt.paintUnactiveText(g2d, bom.getText(), 0, 0);
        
        if(currentStep.x>0)
        {    
            //3 PINTAMOS LOS PATTERNS Y LAS TABLAS
            paintPatternAndTable();
            
            pt.setTextActiveColor(new Color(133,168,234));
            //4 PINTAMOS LAS COINCIDENCIAS EN PATTERN Y TEXT.
            paintPatternAndTextCoincidences();
        }
        //5 PINTAMOS LOS SUBPASOS DEL PASO ACTUAL (miramos que no este complete o vaco)
        if(currentStep.y > -1)
        {
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            paintSubstep();
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
        }
        strategy.show();  //Necesario para mostrar los cambios
    }
    /** Pinta el subpaso actual del algoritmo BOM. */
    private void paintSubstep(){
        
        Point p = (Point)(bomList.get(currentStep.x));
        
        g2d.setStroke(new BasicStroke(1.0f));
        //1 PINTAMOS EL PATTERN CON EL QUE TRABAJAMOS EN LA POSICIN QUE TOCA CON SU TABLA
        pt.drawTable(g2d, currentStep.x+1, p.x, 1, pattern.length()); 
        g2d.setStroke(new BasicStroke(2.0f));
        pt.paintUnactiveText(g2d, pattern, currentStep.x+1, p.x);
        
        //2 PINTAMOS EL CUADRO PARA DEJAR CLARO DONDE ESTAMOS TRABAJANDO
        pt.drawRoundRect(g2d, Color.black, 0                , p.x, 
                                    currentStep.x + 1, p.x + pattern.length() - 1);
        int endCol = p.x + pattern.length();
        String s = bom.getText().substring(endCol - p.y, endCol);
        int coincidences = 0;
        
        if(currentStep.y > 0 && currentStep.y <= p.y)
        {
            //3 PINTAMOS TODAS LAS COINCIDENCIAS COMO ACTIVAS
            coincidences = currentStep.y;
            paintFoundStringInPattern(s, coincidences, currentStep.x + 1, endCol, pt.getTextActiveColor());
            
            //PINTAMOS LA SELECCIN ACTIVA DEL TEXTO
            s = bom.getText().substring(endCol - coincidences, endCol);
            pt.drawTableWithInsideColor(g2d, pt.getTextActiveColor(), 0, endCol - coincidences, 1, coincidences);
            pt.paintText(g2d, s, Color.white, 0, endCol - coincidences);
        }
        if(currentStep.y > 1) 
        {
            //4 PINTAMOS TODAS LAS COINCIDENCIAS EXCEPTO LA LTIMA COMO SELECIONADAS
            coincidences = currentStep.y - 1;
            paintFoundStringInPattern(s, coincidences, currentStep.x + 1, endCol, pt.getTextSelectedColor());
            
            //PINTAMOS LA SELECCIN ENCONTRADA EN EL TEXTO
            s = bom.getText().substring(endCol - coincidences, endCol);
            pt.drawTableWithInsideColor(g2d, pt.getTextSelectedColor(), 0, endCol - coincidences, 1, coincidences);
            pt.paintText(g2d, s, Color.white, 0, endCol - coincidences);
        }
        if(currentStep.y > p.y)
        {
            if(p.y != pattern.length())
            {    
                //5 PINTAMOS EL CARACTER NO ENCONTRADO EN ESTE PASO
                s = bom.getText().substring(endCol - currentStep.y, endCol - currentStep.y + 1 );
                
                pt.drawTableWithInsideColor(g2d, Color.red, 0, endCol - currentStep.y, 1, 1);
                pt.paintText(g2d, s, Color.white, 0, endCol - currentStep.y);
                
                //RECTNGULO 
                g2d.setStroke(new BasicStroke(3.0f));
                pt.drawRoundRect(g2d, Color.white, 0                , p.x, 
                                        currentStep.x + 1, p.x + pattern.length() - 1);
                pt.drawRoundRect(g2d, Color.red, 0                , p.x, 
                                        currentStep.x + 1, p.x + pattern.length() - 1);
            }
            else //Hemos encontrado todo el pattern
            {
                coincidences = currentStep.y - 1;
                
                //paintFoundStringInPattern(pattern, coincidences, currentStep.x + 1, endCol, pt.getTextActiveColor());
                pt.drawTableWithInsideColor(g2d, pt.getTextActiveColor(), currentStep.x +1, endCol - coincidences, 1, coincidences);
                pt.paintText(g2d, pattern, Color.white, currentStep.x+1, endCol - coincidences);
                
                pt.drawTableWithInsideColor(g2d, pt.getTextActiveColor(), 0, endCol - coincidences, 1, coincidences);
                pt.paintText(g2d, pattern, Color.white, 0, endCol - coincidences);
                                
                g2d.setStroke(new BasicStroke(3.0f));
                pt.drawRoundRect(g2d, Color.white, 0                , p.x, 
                                        currentStep.x + 1, p.x + pattern.length() - 1);
                pt.drawRoundRect(g2d, Color.black, 0                , p.x, 
                                        currentStep.x + 1, p.x + pattern.length() - 1);
            }
        }
    }
    /** Resetea las variables de MyCanvasBOM independientes del pattern y el text. */
    private void resetParameters(){
        
        leftButtonClicked = false;
        currentStep = new Point(0, -1);
        moving = true;
    }
    private void initGraphics2D(){
        
        createBufferStrategy(2);
        strategy = getBufferStrategy();
        g2d = (Graphics2D)strategy.getDrawGraphics();
        //Le damos valores al render de g
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING  , RenderingHints.VALUE_COLOR_RENDER_SPEED);
        g2d.setRenderingHint(RenderingHints.KEY_DITHERING        , RenderingHints.VALUE_DITHER_ENABLE );
        g2d.setRenderingHint(RenderingHints.KEY_RENDERING        , RenderingHints.VALUE_RENDER_SPEED);
        g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
    }
    private Point getInitialPoint(){
        
        return new Point( 30, 15 );
    }
    /** Pinta todo el Canvas con el color de fondo indicado por el PaintGraphBOM. */ 
    protected void cleanBackground(Color c){
        
        //Rectngulo de Fondo//////////////
        Point p = pt.getTextStartPoint();
        Object antialiasingMode = g2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        //Si nos movemos hay que pintar de nuevo el fondo y las sombras
        if(moving) super.cleanBackground(c);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        pt.drawBackgroundRect(g2d, p.x-20, p.y-10, backgroundRectangleSize, moving);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasingMode);
        moving = false;
    }
    /** Realiza una bsqueda en GraphOracle(pattern) del String para ver en que posiciones se ha encontrado
     *  y lo pinta. Por ejemplo, si buscamos ATGTG en el pattern ATGTATG (GraphOracle asociado
     *  GTATGTA) pintar ATGtaTG.(Maysculas equivalen a pintado).
     *  @param found String que buscaremos para pintar.
     *  @param coincidences el n de caracteres que queremos buscar para pintar.
     *  @param row La fila donde queremos empezar a pintar el String.
     *  @param col La columna donde queremos empezar a pintar el String (de derecha a izqda).
     *  @param c Color con el que pintatemos el texto */
    private void paintFoundStringInPattern(String found, int coincidences, int row, int col, Color c){
    
        int node = 0;
        int nodeDesti = 0;
        String aux = "";
    
        for(int j = found.length(); j>found.length() - coincidences; j--)
        {
            aux = found.substring(j-1, j); 
            nodeDesti = fo.getNode(node).hasTransition(aux);
            pt.paintText(g2d, aux, c, row, col - nodeDesti);
            node = nodeDesti;
        }
    }
    /** Pinta los patterns en sus correspondientes posiciones y las tablas que los rodean siguiendo
     *  el contenido del bomList. */
    private void paintPatternAndTable(){
        
        Point p; 
        
        //PRIMERO LOS PATTERNS CON SUS TABLE
        for(int i = 0; i<currentStep.x; i++)
        {
            p = (Point)bomList.get(i);
            //Pintamos el pattern donde toca
            pt.paintUnactiveText(g2d, pattern, i+1, p.x);
            pt.drawTable(g2d, i+1, p.x, 1, pattern.length());            
        }
    }
    /** Pinta los recuadros entre el texto y el pattern all donde se han encontrado coincidencias
     *  pero no el pattern entero. Tambin pinta las coincidencias correspondientes en el texto
     *  y pattern. */
    private void paintPatternAndTextCoincidences(){
        
        int endCol;
        Point p;
        String currentS;
        Color[] c = {Color.red, new Color(0, 180, 0)};
        
        for(int i = 0; i<currentStep.x; i++)
        {
            p = ((Point)bomList.get(i));
            //Pintamos las coincidencias en el texto y los patterns
            if(p.y == pattern.length()) //hemos encontrado el pattern completo
            {
                endCol = p.x + pattern.length();
                currentS = bom.getText().substring(p.x, endCol);
                
                pt.drawTableWithInsideColor(g2d, pt.getTextActiveColor(), 0, p.x, 1, pattern.length());
                pt.paintText(g2d, currentS, Color.white, 0, p.x);
                pt.paintActiveText(g2d, currentS, i+1, p.x);
            }
            else if(p.y > 0) //hemos encontrado un factor del pattern
            {
                endCol = p.x + pattern.length();
                currentS = bom.getText().substring(endCol - p.y, endCol);
                
                //pt.drawTableWithInsideColor(g2d, pt.getTextSelectedColor(), 0, endCol - p.y, 1, p.y);
                //pt.paintText(g2d, currentS, Color.white, 0, endCol - p.y);
                paintFoundStringInPattern(currentS, p.y, i+1, endCol, pt.getTextSelectedColor());
            } 
        }
    } 
}