/*
 * PaintGraphBOM.java
 *
 * Created on 19 de marzo de 2005, 17:19
 */

package oracle.paint;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import oracle.Transition;
import oracle.FactorOracle;

import oracle.FactorOracleConst;

/**
 *
 * @author Alejandro Touza
 */
public class PaintFactorOracle extends PaintAutomaton{
    
    protected Color linkColor;
    protected Color linkSelectedColor;
    protected Color linkCreatedColor;
    protected int   linkHeight;
    protected int   linkStart;
    protected float linkStroke;
    
    // CONSTRUCTORAS
    /** Creates a new instance of PaintGraphBOM */
    public PaintFactorOracle() {
        
        setDefaultValues();
    }
    
    //CONSULTORAS
    public Color getLinkColor(){
        
        return linkColor;
    }
    public Color getLinkSelectedColor(){
        
        return linkSelectedColor;
    }
    public Color getLinkCreatedColor(){
        
        return linkCreatedColor;
    }
    public int getLinkHeight(){
        
        return linkHeight;
    }
    public int getLinkStart(){
        
        return linkStart;
    }
    public float getLinkStroke(){
        
        return linkStroke;
    }
    
    //MODIFICADORAS
    public void setDefaultValues(){
        
        super.setDefaultValues();
        linkColor         = FactorOracleConst.LINK_COLOR;
        linkCreatedColor  = FactorOracleConst.LINK_CREATED_COLOR;
        linkSelectedColor = FactorOracleConst.LINK_SELECTED_COLOR;
        linkHeight        = FactorOracleConst.LINK_HEIGHT;
        linkStart         = FactorOracleConst.LINK_START;
        linkStroke        = FactorOracleConst.LINK_STROKE;
    }
    public void setLinkColor(Color c){
        
        linkColor = c;
    }
    public void setLinkSelectedColor(Color c){
      
        linkSelectedColor = c;
    }
    public void setLinkCreatedColor(Color c){
      
        linkCreatedColor = c;
    }
    public void setLinkHeight(int i){
      
        linkHeight = i;
    }
    public void setLinkStart(int i){
      
        linkStart = i;
    }
    public void setLinkStroke(float f){
      
        linkStroke = f;
    }
    public void resize(Dimension d){
        super.resize(d);
        this.linkHeight = transitionHeight;
    }
    
    //Pintado
    /** Pinta por completo el Graph Oracle go con los parmetros que tengamos
     *  definidos en PaintGraphOracle. */ 
    public void paintFactorOracle(Graphics2D g, FactorOracle go, boolean drawLink){
        
        if(drawLink)
        {
            for(int i=0; i<go.size();i++)
            {
                for(int j=0; j<go.getNode(i).getNumberOfTransitions(); j++)
                    super.drawTransition(g, go.getNode(i).getTransition(j), false);
                drawLink(g, go.getNode(i).getLink());
            }
            for(int i=0; i<go.size();i++) super.paintNode(g, go.getNode(i));
        }
        else
        {
            for(int i=0; i<go.size();i++)
            {
                for(int j=0; j<go.getNode(i).getNumberOfTransitions(); j++)
                    super.drawTransition(g, go.getNode(i).getTransition(j), false);
                //Podemos pintar los nodos en el bucle pq no tendremos suffix-link
                //que se pinten por encima de los nodos.
                super.paintNode(g, go.getNode(i));
            }
        }
    }
    /** Pinta por completo el Graph Oracle go con los parmetros que tengamos
     *  definidos en PaintGraphOracle */ 
    public void paintRoughFactorOracle(Graphics2D g, FactorOracle go, boolean drawLink){
        
        g.setStroke(new BasicStroke(1f));
     //g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
        if(drawLink) for(int i=0; i<go.size();i++) drawRoughLink(g, go.getNode(i).getLink());
        for(int i=0; i<go.size();i++)
        {
            for(int j=0; j<go.getNode(i).getNumberOfTransitions(); j++)
                super.drawRoughTransition(g, go.getNode(i).getTransition(j), false);
            super.paintRoughNode(g, go.getNode(i));
        }
     //g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    }
   /** Dibuja los suffix-link del grafo que siempre van de un nodo cualquiera 
   *  a uno anterior con Color c.*/
   public void drawLink(Graphics2D g, Transition t, Color c, float stroke){
     
       int distanciaEntreNodos;
       
       if ( (distanciaEntreNodos = Math.abs(t.getStart() - t.getEnd())) == 0 ) return; 
       
       g.setStroke(new BasicStroke(stroke));
       ////////////////////////////////////////////////////////////////////////////////////
       // Sabemos que los suffix-link van siempre de un nodo posterior a uno anterior
       // as que tampoco comprobamos eso
       ////////////////////////////////////////////////////////////////////////////////////
       // h:  Altura a la que llegar nuestra transicin (dependiente de la distancia entre
       //     los nodos inicial y final de la misma). El -1 es porque si los nodos son 
       //     consecutivos dibujaremos una flecha entre ellos en lugar de un arco.
       // xA: Coordenada x donde empieza la caja que contiene el arco (o flecha) 
       //     a dibujar (es la posicin donde empieza el nodo + 1/2 de la anchura del nodo).
       // yA: Coordenada y donde empieza la caja que contiene el arco a dibujar 
       //     (tiene en cuenta la distancia en nodos para no cruzar a otros posibles arcos).
       // xW: Anchura de la caja que contiene el arco a dibujar (es la distancia 
       //     horizontal desde un nodo hasta el nodo destino).
       // yH: Altura de la caja que contiene el arco a dibujar (es el doble de la distancia
       //     desde yA hasta el centro del nodo + la mitad de la altura del nodo + altura
       //     extra h en funcin de la distancia entre los nodos de la transicin para evitar
       //     cruces con otras transiciones). 
       //
       int h  = transitionHeight * distanciaEntreNodos;
       int xA = automatonStartPoint.x + t.getEnd()*(nodeTotalWidth) + nodeSize.width/2;
       int yA = automatonStartPoint.y - h - transitionStart;
       int xW = distanciaEntreNodos * nodeTotalWidth;
       int yH = 2 * (transitionStart + h + nodeSize.height/2);
       ////////////////////////////////////////////////////////////////////////////////////
       
       g.setColor(c);
       g.drawArc(xA, yA, xW, yH, 0, GRADOS_ARCO);
       super.drawArrow(g, xA+xW/2+2, yA, xA+xW/2-2, yA, transitionArrowSize);
   }
   
 /** Dibuja los suffix-link del grafo que siempre van de un nodo cualquiera 
   *  a uno anterior. No pinta la flecha y usa el color por defecto del suffixLink. */
   public void drawRoughLink(Graphics2D g, Transition t){
      
       int distanciaEntreNodos;
       
       if ( (distanciaEntreNodos = Math.abs(t.getStart() - t.getEnd())) == 0 ) return;  
       
       int h  = transitionHeight * distanciaEntreNodos;
       int xA = automatonStartPoint.x + t.getEnd()*(nodeTotalWidth) + nodeSize.width/2;
       int yA = automatonStartPoint.y - h - transitionStart;
       int xW = distanciaEntreNodos * nodeTotalWidth;
       int yH = 2 * (transitionStart + h + nodeSize.height/2);
       
       g.setColor(linkColor);
       g.drawArc(xA, yA, xW, yH, 0, GRADOS_ARCO);
   }
   
   /** Dibuja los suffix-link del grafo que siempre van de un nodo cualquiera 
    *  a uno anterior con el color por defecto de linkColor. */
   public void drawLink(Graphics2D g, Transition t){
      
       drawLink(g, t, linkColor, linkStroke);
   }
   /** Dibuja los suffix-link del grafo que siempre van de un nodo cualquiera 
    *  a uno anterior con el color por defecto de linkSelectedColor. */
   public void drawSelectedLink(Graphics2D g, Transition t){
      
       drawLink(g, t, linkSelectedColor, linkStroke);
   }
   /** Dibuja los suffix-link del grafo que siempre van de un nodo cualquiera 
    *  a uno anterior con el color por defecto de linkCreatedColor. */
   public void drawCreatedLink(Graphics2D g, Transition t){
      
       drawLink(g, t, linkCreatedColor, linkStroke);
   }
  
}
