/*
 * SimpleLinkListener.java
 * A hyperlink listener for use with JEditorPane.  This
 * listener will change the cursor over hotspots based on enter/exit
 * events and also load a new page when a valid hyperlink is clicked.
 */

package showHTML;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

public class SimpleLinkListener implements HyperlinkListener {

  private JEditorPane pane;       // The pane we're using to display HTML

  public SimpleLinkListener(JEditorPane jep) {
    pane = jep;
  }

  public void hyperlinkUpdate(HyperlinkEvent he) {
    
    HyperlinkEvent.EventType type = he.getEventType();
    // Ok.  Decide which event we got...
    if (type == HyperlinkEvent.EventType.ENTERED) {
      // Enter event.  
      pane.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }
    else if (type == HyperlinkEvent.EventType.EXITED) {
      // Exit event.  
      pane.setCursor(Cursor.getDefaultCursor());
    }
    else {
      // Jump event.  Get the url, and if it's not null, switch to that
      // page in the main editor pane
      try {
	pane.setPage(he.getURL());
      }
      catch (Exception e) {
	e.printStackTrace();
      }
    }
  }
}

